// Simple TOF-to-energy conversion (CLASSICAL)
double TOFtoE_classical(double t_flash, double tof) {
	double	t_light  = 0.;
	double	n_energy = 0.;

	t_light  = L_geom / C;
	n_energy = 0.5 * Mn * (L_geom / (tof - t_flash + t_light)) * (L_geom / (tof - t_flash + t_light)) / (C*C);
	return n_energy;
}

// Simple TOF-to-energy conversion (RELATIVISTIC)
double TOFtoE_relativistic(double t_flash, double tof) {
	double beta     = 0.;
	double ggg      = 0.; // = 1 / gamma
	double gamma    = 0.;
	double t_light  = 0.;
	double n_energy = 0.;
	
	t_light  = L_geom / C;
	beta     = (double) L_geom / (tof - t_flash + t_light) / C;
	ggg      = sqrt((1.-beta*beta));
	gamma    = (double) 1. / ggg;
	n_energy = (gamma - 1.) * Mn; // energy in eV
	return n_energy;
}

// Energy-to-TOF conversion (RELATIVISTIC)
double Energy_to_Time_relativistic(double E){
	double t = 0.;
	t = L_geom/(C*sqrt(1.-1./pow (1.+(E/Mn),2)));
	return t;
}

// n2f factor calculation
double neutron_to_flux(double binning){
	double stepsize = 0.;
	double s        = 0.;
	double n_to_f   = 0.;

	stepsize = 1./binning;
	s        = pow(10,stepsize);
	n_to_f   =  sqrt(s)/(s - 1.);
	return n_to_f;
}

// Full TOF-to-energy conversion using moderation length data (CLASSICAL)
double TOFtoE_classical_lamda(double t_flash, double tof) {
	double	t_light  = 0.;
	double	n_energy = 0.;

	t_light  = L_geom / C;
	n_energy = 0.5 * Mn * (L_geom / (tof - t_flash + t_light)) * (L_geom / (tof - t_flash + t_light)) / (C*C);

	// Determine lamda bin
	double	log_en = 0.;	// logarithm of the energy
	double	energy[10] = {0.};

	energy[0] = n_energy;

	for (j=0; ; j++) {
		int	l = 0;
		log_en = log10(energy[j]);
		l = (int)((log_en+3)*10 + 1);	// log_en-(-3))/0.1 Minimum energy 1.0E-3 eV, 10 BPD (up to 10 GeV)
		if(l < 0 || l > 130) {
			break;
		}

		energy[j+1] = 0.5 * Mn * ((L_geom + lamda[l]) / (tof - t_flash + t_light)) * ((L_geom + lamda[l]) / (tof - t_flash + t_light)) / (C*C);
		if(100. * fabs((energy[j+1]-energy[j])/energy[j]) < 0.1) {	// if new energy value within 0.1% of previous...
			n_energy = energy[j];
			break;
			}
	}

	return n_energy;
}

// Full TOF-to-energy conversion using moderation length data (RELATIVISTIC)
double TOFtoE_relativistic_lamda(double t_flash, double tof) {
	double	beta     = 0.;
	double	ggg      = 0.;		// = 1 / gamma
	double	gamma    = 0.;
	double	t_light  = 0.;
	double	n_energy = 0.;

	t_light  = L_geom / C;
	beta     = (double) L_geom / (tof - t_flash + t_light) / C;
	ggg      = sqrt((1.-beta*beta));
	gamma    = (double) 1. / ggg;
	n_energy = (gamma - 1.) * Mn; // energy in eV

	// Determine lamda bin
	double	log_en = 0.;	// logarithm of the energy
	double	energy[10] = {0.};

	energy[0] = n_energy;

	for (j=0; ; j++) {
		int	l = 0;
		log_en = log10(energy[j]);
		l = (int)((log_en+3)*10 + 1);	// log_en-(-3))/0.1 Minimum energy 1.0E-3 eV, 10 BPD (up to 10 GeV)
		if(l < 0 || l > 130) {
			break;
		}

		beta        = (double) (L_geom + lamda[l]) / (tof - t_flash + t_light) / C;
		ggg         = sqrt((1.-beta*beta));
		gamma       = (double) 1. / ggg;
		energy[j+1] = (gamma - 1.) * Mn; // energy in eV
		if(100. * fabs((energy[j+1]-energy[j])/energy[j]) < 0.1) {	// if new energy value within 0.1% of previous...
			n_energy = energy[j];
			break;
			}
	}

	return n_energy;
}
